package com.yonge.cooleshow.biz.dal.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.biz.dal.enums.*;
import com.yonge.toolset.payment.base.model.callback.ExecutePaymentCallBack;
import com.yonge.toolset.payment.util.DistributedLock;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.common.constant.SysConfigConstant;
import com.yonge.cooleshow.biz.dal.dto.req.OrderPayReq;
import com.yonge.cooleshow.biz.dal.dto.req.OrderReq;
import com.yonge.cooleshow.biz.dal.entity.UserOrder;
import com.yonge.cooleshow.biz.dal.entity.UserOrderPayment;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.res.OrderCreateRes;
import com.yonge.cooleshow.biz.dal.vo.UserOrderVo;
import com.yonge.cooleshow.biz.dal.vo.res.OrderPayRes;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.CacheNameEnum;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.cooleshow.common.service.IdGeneratorService;
import com.yonge.toolset.base.result.BaseResult;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.payment.base.enums.OpenEnum;
import com.yonge.toolset.payment.base.enums.PaymentClientEnum;
import com.yonge.toolset.payment.base.enums.TradeStatusEnum;
import com.yonge.toolset.payment.base.model.Payment;
import com.yonge.toolset.payment.base.model.callback.PaymentCallBack;
import com.yonge.toolset.payment.core.props.PaymentProperties;
import com.yonge.toolset.payment.core.service.PaymentClient;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yonge.cooleshow.biz.dal.dto.search.OrderSearch;
import com.yonge.cooleshow.biz.dal.dao.UserOrderDao;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import java.util.function.Function;

@Service
public class UserOrderServiceImpl extends ServiceImpl<UserOrderDao, UserOrder> implements UserOrderService {
    private final static Logger log = LoggerFactory.getLogger(UserOrderServiceImpl.class);

    @Autowired
    private MemberPriceSettingsService memberPriceSettingsService;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private UserOrderDetailService orderDetailService;
    @Autowired
    private UserOrderPaymentService orderPaymentService;
    @Autowired
    private PaymentClient paymentClient;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private CourseGroupService courseGroupService;
    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private VideoLessonPurchaseRecordService recordService;
    @Autowired
    private MusicSheetService musicSheetService;
    @Autowired
    private PianoRoomBuyRecordService pianoRoomBuyRecordService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private PaymentProperties paymentProperties;

    //验证订单是否可以下单,获取订单金额信息
    private static final Map<GoodTypeEnum, Function<OrderReq.OrderReqInfo, HttpResponseResult<OrderCreateRes>>> orderCreate = new HashMap<>();
    //插入订单后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> orderAfter = new HashMap<>();
    //订单完成后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> orderSuccess = new HashMap<>();
    //订单取消后执行
    private static final Map<GoodTypeEnum, Consumer<UserOrderDetailVo>> orderCancel = new HashMap<>();

    @PostConstruct
    private void init() {
        /**********订单生成前******************/
        //vip开通缴费
        orderCreate.put(GoodTypeEnum.VIP, memberPriceSettingsService::orderCreate);
        //直播课程购买
        orderCreate.put(GoodTypeEnum.LIVE, courseGroupService::buyLiveCourse);
        //陪练课购买
        orderCreate.put(GoodTypeEnum.PRACTICE, scheduleService::buyPracticeCourse);
        //视频课购买
        orderCreate.put(GoodTypeEnum.VIDEO, recordService::buyVideoCourse);
        //曲目购买
        orderCreate.put(GoodTypeEnum.MUSIC, musicSheetService::buyMusicSheetCheck);
        //琴房时长
        orderCreate.put(GoodTypeEnum.PINAO_ROOM, pianoRoomBuyRecordService::orderCreate);

        /**********订单生成后******************/


        //曲目购买
        orderAfter.put(GoodTypeEnum.MUSIC, musicSheetService::orderAfterSheet);

        /**********订单完成后******************/
        //vip开通缴费
        orderSuccess.put(GoodTypeEnum.VIP, memberPriceSettingsService::orderSuccess);
        //直播课程购买
        orderSuccess.put(GoodTypeEnum.LIVE, courseGroupService::buyLiveCourseSuccess);

        //陪练课购买
        orderSuccess.put(GoodTypeEnum.PRACTICE, scheduleService::buyPracticeCourseSuccess);
        //视频课购买
        orderSuccess.put(GoodTypeEnum.VIDEO, recordService::buyVideoCourseSuccess);

        //曲目购买
        orderSuccess.put(GoodTypeEnum.MUSIC, musicSheetService::buyMusicSheetSuccess);

        //琴房时长
        orderSuccess.put(GoodTypeEnum.PINAO_ROOM, pianoRoomBuyRecordService::orderSuccess);

        /**********订单取消后******************/
        //陪练课购买
        orderCancel.put(GoodTypeEnum.PRACTICE, scheduleService::buyPracticeCourseFailed);
        //视频课购买
        orderCancel.put(GoodTypeEnum.VIDEO, recordService::buyVideoCourseFailed);
        //直播课购买
        orderCancel.put(GoodTypeEnum.LIVE, courseGroupService::buyLiveCourseCancel);

        // 曲目购买
        orderCancel.put(GoodTypeEnum.MUSIC, musicSheetService::buyMusicSheetCancel);
    }

    @Override
    public UserOrderVo detail(Long id) {
        UserOrderVo userOrderVo = baseMapper.detailById(id);
        if (null != userOrderVo) {
            userOrderVo.setOrderDetailList(orderDetailService.getOrderDetilListByOrderNo(userOrderVo.getOrderNo()));
        }
        return userOrderVo;
    }

    @Override
    public UserOrderVo detail(String orderNo, Long userId) {
        UserOrder param = new UserOrder();
        param.setUserId(userId);
        param.setOrderNo(orderNo);
        UserOrderVo userOrderVo = baseMapper.detailByOrderNo(param);
        if (null != userOrderVo) {
            userOrderVo.setOrderDetailList(orderDetailService.getOrderDetilListByOrderNo(userOrderVo.getOrderNo()));
        }
        return userOrderVo;
    }

    @Override
    public UserOrderVo detailApp(UserOrder param) {
        UserOrderVo userOrderVo = baseMapper.detailApp(param);
        if (null != userOrderVo) {
            userOrderVo.setFeeAmt(null);
            userOrderVo.setPlantformFee(null);
            List<UserOrderDetailVo> orderDetilList = orderDetailService.getOrderDetilListByOrderNo(userOrderVo.getOrderNo());
            orderDetilList.forEach(o -> {
                o.setPlantformFee(null);
                o.setPlantformFeeRate(null);
            });
            userOrderVo.setOrderDetailList(orderDetilList);
        }
        return userOrderVo;
    }

    @Override
    public UserOrderVo getUserOrderByPaymentNoOrTransNo(String paymentNo, String transNo) {
        return baseMapper.getUserOrderByPaymentNoOrTransNo(paymentNo, transNo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<Boolean> orderCancel(OrderPayReq payReq) {
        UserOrderVo detail = detail(payReq.getOrderNo(), payReq.getUserId());
        if (null == detail) {
            return HttpResponseResult.failed("未找到订单信息");
        }
        if (!OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                && !OrderStatusEnum.PAYING.equals(detail.getStatus())) {
            return HttpResponseResult.failed("订单已关闭");
        }
        //关闭或完成订单
        closeOrSuccessOrder(payReq.getOrderNo());
        return HttpResponseResult.succeed(true);
    }


    @Override
    public void setSuccessStatus() {
        OrderSearch query = new OrderSearch();
        query.setStatus("WAIT_PAY,PAYING");
        List<UserOrderVo> userOrderVos = selectAllList(query);
        for (UserOrderVo userOrderVo : userOrderVos) {
            testOrderSuccess(userOrderVo);
        }
    }

    @Override
    public List<UserOrder> selectUnRecordTimeOrder(OrderSearch query) {
        return baseMapper.selectUnRecordTimeOrder(query);
    }

    @Override
    public Integer updateStatusByOrderNo(String orderNo, String orderStatus) {
        return baseMapper.updateStatusByOrderNo(orderNo, orderStatus);
    }

    @Override
    public void setOrderStatus(String orderNo, OrderStatusEnum orderStatus) {
        UserOrderVo detail = detail(orderNo, null);
        if (OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                || OrderStatusEnum.PAYING.equals(detail.getStatus())) {

            if (OrderStatusEnum.CLOSE.equals(orderStatus)) {
                doOrderCancel(detail, orderStatus, "测试接口-订单关闭");
            }
            if (OrderStatusEnum.FAIL.equals(orderStatus)) {
                doOrderCancel(detail, orderStatus, "测试接口-订单交易失败");
            }
            if (OrderStatusEnum.PAID.equals(orderStatus)) {
                testOrderSuccess(detail);
            }
        }
    }


    @Override
    public IPage<UserOrderVo> selectPage(IPage<UserOrderVo> page, OrderSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    @Override
    public List<UserOrderVo> selectAllList(OrderSearch query) {
        return baseMapper.selectAllList(query);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<UserOrder> executeOrder(OrderReq orderReq) {
        long start = System.currentTimeMillis();
        log.info("订单[创建订单] Req：{}", JSONObject.toJSONString(orderReq));
        //订单号生成
        Long orderNo = idGeneratorService.generatorId("orderNo");
        orderReq.setOrderNo(Long.toString(orderNo));
        //验证参数,必须验证参数
        BigDecimal actualPrice = BigDecimal.ZERO;
        for (OrderReq.OrderReqInfo info : orderReq.getOrderInfos()) {
            Function<OrderReq.OrderReqInfo, HttpResponseResult<OrderCreateRes>> createFunction = orderCreate.get(info.getGoodType());
            if (Objects.isNull(createFunction)) {
                return HttpResponseResult.failed("订单商品信息获取失败");
            }
            info.setOrderNo(orderReq.getOrderNo());
            Long subOrderNo = idGeneratorService.generatorId("subOrderNo");
            info.setSubOrderNo(Long.toString(subOrderNo));

            info.setUserId(orderReq.getUserId());
            HttpResponseResult<OrderCreateRes> createResult = createFunction.apply(info);
            OrderCreateRes createRes = createResult.getData();
            if (!createResult.getStatus() || null == createRes || !createRes.getRes()) {
                log.info("订单[创建订单] Res：{}", JSONObject.toJSONString(createResult));
                return HttpResponseResult.failed(StringUtil.isEmpty(createResult.getMsg()) ? "订单商品信息获取失败" : createResult.getMsg());
            }
            createRes.setGoodType(info.getGoodType());
            //todo 优惠券优惠金额，暂时为0
            BigDecimal couponAmount = BigDecimal.ZERO;
            createRes.setActualPrice(createRes.getExpectPrice().subtract(couponAmount).setScale(2, RoundingMode.HALF_UP));
            createRes.setCouponAmount(couponAmount);
            //累计实际支付金额
            actualPrice = actualPrice.add(createRes.getActualPrice());

            info.setCreateRes(createRes);
        }

        //验证金额
        if (actualPrice.setScale(2, RoundingMode.HALF_UP)
                .compareTo(orderReq.getActualPrice().setScale(2, RoundingMode.HALF_UP)) != 0) {
            return HttpResponseResult.failed("订单金额校验不通过");
        }

        //验证成功后，订单入库
        UserOrderVo orderVo = insertOrder(orderReq);

        List<UserOrderDetailVo> orderDetailList = orderVo.getOrderDetailList();
        for (UserOrderDetailVo orderDetailVo : orderDetailList) {
            orderDetailVo.setUserId(orderReq.getUserId());
            orderDetailVo.setOrderId(orderVo.getId());

            Consumer<UserOrderDetailVo> afterFunction = orderAfter.get(orderDetailVo.getGoodType());
            if (!Objects.isNull(afterFunction)) {
                afterFunction.accept(orderDetailVo);
            }
        }
        log.info("下单请求 start is {} end is {}", start, System.currentTimeMillis());

        //判断订单是否是无需支付的订单，若是，则直接完成订单
        if (BigDecimal.ZERO.compareTo(orderVo.getActualPrice()) == 0) {
            orderSuccess(orderVo);
        }
        return HttpResponseResult.succeed(orderVo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<OrderPayRes> orderPay(OrderPayReq payReq) {
        //查询订单
        UserOrderVo detail = detail(payReq.getOrderNo(), payReq.getUserId());
        if (null == detail || !payReq.getUserId().equals(detail.getUserId())) {
            return HttpResponseResult.failed("订单不存在");
        }
        //查询付款单
        if (!OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                && !OrderStatusEnum.PAYING.equals(detail.getStatus())) {
            return HttpResponseResult.failed("订单状态异常");
        }
        try {
            return doOrderPay(payReq, detail);
        } catch (BizException e) {
            return HttpResponseResult.failed(e.getMessage());
        } catch (Exception e) {
            return HttpResponseResult.failed("付款失败");
        }
    }

    @Override
    public HttpResponseResult<UserOrderVo> getPendingOrder(OrderSearch query) {
        UserOrderVo userOrderVo = baseMapper.getPendingOrder(query);
        if (null != userOrderVo) {
            userOrderVo.setOrderDetailList(orderDetailService.getOrderDetilListByOrderNo(userOrderVo.getOrderNo()));
        }
        return HttpResponseResult.succeed(userOrderVo);
    }

    @Override
    public void pollingOrder() {
        //查询半小时前创建的并且还未完成的订单 WAIT_PAY 待支付订单 PAYING 支付中订单
        OrderSearch search = new OrderSearch();
        search.setStatus(OrderStatusEnum.WAIT_PAY.getCode());
        search.setEndTime(LocalDateTime.now().minusMinutes(30));
        //查询待支付订单
        List<UserOrderVo> waitPayOrderList = baseMapper.selectAllList(search);
        for (UserOrderVo orderVo : waitPayOrderList) {
            //取消订单要获取锁，防止在轮询取消时，用户修改订单状态
            boolean b = DistributedLock.of(redissonClient)
                    .runIfLockCanGet(CacheNameEnum.LOCK_EXECUTE_ORDER.getRedisKey(orderVo.getOrderNo())
                            , () -> pollingCancelOrder(orderVo.getOrderNo()), 60L, TimeUnit.SECONDS);
        }

        search.setStatus(OrderStatusEnum.PAYING.getCode());
        search.setEndTime(LocalDateTime.now().minusMinutes(120));
        //查询待支付中订单
        List<UserOrderVo> payingOrderList = baseMapper.selectPendingList(search);
        for (UserOrderVo orderVo : payingOrderList) {
            //取消订单要获取锁，防止在轮询取消时，用户修改订单状态
            boolean b = DistributedLock.of(redissonClient)
                    .runIfLockCanGet(CacheNameEnum.LOCK_EXECUTE_ORDER.getRedisKey(orderVo.getOrderNo())
                            , () -> pollingCancelOrder(orderVo.getOrderNo()), 60L, TimeUnit.SECONDS);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    void pollingCancelOrder(String orderNo) {
        closeOrSuccessOrder(orderNo);
    }

    private void closeOrSuccessOrder(String orderNo) {
        UserOrderVo detail = detail(orderNo, null);
        if (null == detail) {
            return;
        }
        if (!OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                && !OrderStatusEnum.PAYING.equals(detail.getStatus())) {
            return;
        }
        UserOrderPayment userOrderPayment = orderPaymentService.detailByTransNoOrPaymentNo(detail.getTransNo(), detail.getPaymentNo());
        if (null == userOrderPayment) {
            //关闭订单
            doOrderCancel(detail, OrderStatusEnum.CLOSE, "订单超时");
        } else if (TradeStatusEnum.pending.equals(userOrderPayment.getStatus())) {
            Payment param = new Payment(detail.getOpenType(),
                    PaymentClientEnum.valueOf(userOrderPayment.getPaymentClient()), detail.getPayChannel());
            param.setId(detail.getTransNo());
            param.setPaymentNo(detail.getPaymentNo());
            BaseResult<Payment> res = paymentClient.queryPayment(param);
            Payment queryPaymentData = res.getData();

            //支付成功
            if (res.getStatus() && TradeStatusEnum.succeeded.getCode().equals(queryPaymentData.getStatus())
                    && (OrderStatusEnum.WAIT_PAY.equals(detail.getStatus()) || OrderStatusEnum.PAYING.equals(detail.getStatus()))) {
                //订单已经完成，则完成订单
                orderSuccess(detail);
                //付款单改为已完成状态
                userOrderPayment.setTransNo(queryPaymentData.getId());
                userOrderPayment.setStatus(TradeStatusEnum.succeeded);
                userOrderPayment.setUpdateTime(new Date());
                userOrderPayment.setArrivalTime(new Date());
                userOrderPayment.setBackPayAmt(
                        new BigDecimal(queryPaymentData.getPayAmt()));

                ExecutePaymentCallBack executePaymentCallBack = queryPaymentData.getExecutePaymentCallBack();
                if (null != executePaymentCallBack) {
                    userOrderPayment.setArrivalTime(executePaymentCallBack.getTradeTime());
                    userOrderPayment.setBackPayAmt(
                            new BigDecimal(executePaymentCallBack.getPayAmt()));
                    userOrderPayment.setFeeAmt(new BigDecimal(executePaymentCallBack.getFeeAmt()));
                }
                orderPaymentService.updateById(userOrderPayment);
            } else {
                //关闭订单付款单
                Boolean close = orderPaymentService.closePaymentAndReqOpen(orderNo, "订单超时");
                if (close) {
                    //关闭订单
                    doOrderCancel(detail, OrderStatusEnum.CLOSE, "订单超时");
                }
            }
        } else if (TradeStatusEnum.succeeded.equals(userOrderPayment.getStatus())) {
            //订单已经完成，则完成订单
            orderSuccess(detail);
        } else if (TradeStatusEnum.failed.equals(userOrderPayment.getStatus())) {
            //入订单付款表，同时修改订单状态
            doOrderCancel(detail, OrderStatusEnum.FAIL, userOrderPayment.getPayFailMsg());
        } else if (TradeStatusEnum.close.equals(userOrderPayment.getStatus())) {
            //关闭订单
            doOrderCancel(detail, OrderStatusEnum.CLOSE, "订单超时");
        }
    }

    /**
     * 判断订单是否完成，若订单已完成则完成订单
     *
     * @param orderNo
     * @return 可以继续支付 true
     */
    private Boolean checkAndSuccessOrder(String orderNo) {
        UserOrderVo detail = detail(orderNo, null);
        if (null == detail) {
            return true;
        }
        if (!OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                && !OrderStatusEnum.PAYING.equals(detail.getStatus())) {
            return false;
        }
        UserOrderPayment userOrderPayment = orderPaymentService.detailByTransNoOrPaymentNo(detail.getTransNo(), detail.getPaymentNo());
        if (null == userOrderPayment) {
            return true;
        }

        Payment param = new Payment(detail.getOpenType(),
                PaymentClientEnum.valueOf(userOrderPayment.getPaymentClient()), detail.getPayChannel());
        param.setId(detail.getTransNo());
        param.setPaymentNo(detail.getPaymentNo());
        BaseResult<Payment> res = paymentClient.queryPayment(param);
        Payment queryPaymentData = res.getData();

        //支付成功
        if (res.getStatus() && TradeStatusEnum.succeeded.getCode().equals(queryPaymentData.getStatus())
                && (OrderStatusEnum.WAIT_PAY.equals(detail.getStatus()) || OrderStatusEnum.PAYING.equals(detail.getStatus()))) {
            //订单已经完成，则完成订单
            orderSuccess(detail, null);
            //付款单改为已完成状态
            userOrderPayment.setTransNo(queryPaymentData.getId());
            userOrderPayment.setStatus(TradeStatusEnum.succeeded);
            userOrderPayment.setUpdateTime(new Date());
            userOrderPayment.setArrivalTime(new Date());
            userOrderPayment.setBackPayAmt(
                    new BigDecimal(queryPaymentData.getPayAmt()));

            ExecutePaymentCallBack executePaymentCallBack = queryPaymentData.getExecutePaymentCallBack();
            if (null != executePaymentCallBack) {
                userOrderPayment.setArrivalTime(executePaymentCallBack.getTradeTime());
                userOrderPayment.setBackPayAmt(
                        new BigDecimal(executePaymentCallBack.getPayAmt()));
                userOrderPayment.setFeeAmt(new BigDecimal(executePaymentCallBack.getFeeAmt()));
            }
            orderPaymentService.updateById(userOrderPayment);
            return false;
        }
        return true;
    }

    private HttpResponseResult<OrderPayRes> doOrderPay(OrderPayReq payReq, UserOrderVo detail) throws Exception {
        Boolean aBoolean = checkAndSuccessOrder(payReq.getOrderNo());
        if (aBoolean) {
            return orderPaymentService.createOrderPayment(payReq, detail);
        } else {
            return HttpResponseResult.failed("订单状态异常");
        }

    }

    /***
     * 订单入库
     * @author liweifan
     * @param: orderReq
     * @updateTime 2022/3/31 16:15
     * @return: com.yonge.cooleshow.biz.dal.entity.Order
     */
    private UserOrderVo insertOrder(OrderReq orderReq) {
        UserOrder userOrder = new UserOrder();

        userOrder.setOrderNo(orderReq.getOrderNo());
        userOrder.setUserId(orderReq.getUserId());
        userOrder.setOrderName(orderReq.getOrderName());
        userOrder.setOrderType(orderReq.getOrderType());

        String desc = "";
        if (StringUtil.isEmpty(orderReq.getOrderDesc())) {
            desc += orderReq.getOrderType().getMsg() + "-" + orderReq.getOrderName();
        } else {
            desc = orderReq.getOrderDesc();
        }
        userOrder.setOrderDesc(desc);
        userOrder.setStatus(OrderStatusEnum.WAIT_PAY);
        userOrder.setUserNote(orderReq.getUserNote());
        userOrder.setCreateTime(new Date());

        userOrder.setOriginalPrice(BigDecimal.ZERO);
        userOrder.setExpectPrice(BigDecimal.ZERO);
        userOrder.setActualPrice(BigDecimal.ZERO);
        userOrder.setCouponAmount(BigDecimal.ZERO);
        userOrder.setPlantformFee(BigDecimal.ZERO);
        List<UserOrderDetailVo> orderDetailList = new ArrayList<>();
        for (OrderReq.OrderReqInfo data : orderReq.getOrderInfos()) {
            OrderCreateRes res = data.getCreateRes();

            UserOrderDetailVo orderDetail = new UserOrderDetailVo();
            orderDetail.setOrderNo(data.getOrderNo());
            orderDetail.setSubOrderNo(data.getSubOrderNo());
            orderDetail.setMerchId(res.getMerchId());
            orderDetail.setGoodType(res.getGoodType());
            orderDetail.setBizId(res.getBizId());
            orderDetail.setBizContent(res.getBizContent());
            orderDetail.setOriginalPrice(null != res.getOriginalPrice() ? res.getOriginalPrice() : BigDecimal.ZERO);
            orderDetail.setExpectPrice(null != res.getExpectPrice() ? res.getExpectPrice() : BigDecimal.ZERO);
            orderDetail.setActualPrice(null != res.getActualPrice() ? res.getActualPrice() : BigDecimal.ZERO);
            orderDetail.setCouponAmount(null != res.getCouponAmount() ? res.getCouponAmount() : BigDecimal.ZERO);

            orderDetail.setGoodNum(null != res.getGoodNum() ? res.getGoodNum() : 1);
            //商品单价计算，使用商品原价计算，退费计算公式（退费金额=实际订单金额-（商品单价 * 已经使用商品数））
            //其中，已经使用的商品单价按照原价算
            orderDetail.setGoodPrice(
                    orderDetail.getOriginalPrice().divide(new BigDecimal(orderDetail.getGoodNum()), 2, RoundingMode.HALF_UP));
            //平台服务费
            BigDecimal serviceFeeRate = getServiceFeeRate(res.getGoodType());
            orderDetail.setPlantformFee(userOrder.getExpectPrice().multiply(serviceFeeRate));
            orderDetail.setPlantformFeeRate(serviceFeeRate);
            orderDetail.setBizParam(res.getBizParam());
            orderDetail.setCreateTime(new Date());

            orderReq.getOrderInfos().forEach(o -> {
                if (o.getGoodType().equals(res.getGoodType())) {
                    orderDetail.setGoodName(o.getGoodName());
                    orderDetail.setBizContent(JSONObject.toJSONString(o.getBizContent()));
                }
            });
            orderDetail.setGoodUrl(getGoodUrlByType(res.getGoodType()));

            orderDetailService.save(orderDetail);
            orderDetailList.add(orderDetail);

            userOrder.setOriginalPrice(userOrder.getOriginalPrice().add(orderDetail.getOriginalPrice()));
            userOrder.setExpectPrice(userOrder.getExpectPrice().add(orderDetail.getExpectPrice()));
            userOrder.setActualPrice(userOrder.getActualPrice().add(orderDetail.getActualPrice()));
            userOrder.setCouponAmount(userOrder.getCouponAmount().add(orderDetail.getCouponAmount()));
            userOrder.setPlantformFee(userOrder.getPlantformFee().add(orderDetail.getPlantformFee()));
        }

        //todo 优惠券优惠金额，暂时为0
        BigDecimal couponAmount = BigDecimal.ZERO;
        userOrder.setActualPrice(userOrder.getActualPrice().subtract(couponAmount));
        userOrder.setCouponAmount(userOrder.getCouponAmount().add(couponAmount));

        if (userOrder.getActualPrice().subtract(orderReq.getActualPrice()).abs()
                .compareTo(new BigDecimal("0.01")) > 0) {
            throw new BizException("交易金额异常");
        }

        int insert = baseMapper.insert(userOrder);

        if (insert == 0 || null == userOrder.getId()) {
            throw new BizException("创建订单失败");
        }
        UserOrderVo vo = new UserOrderVo();

        BeanUtils.copyProperties(userOrder, vo);
        vo.setOrderDetailList(orderDetailList);
        return vo;
    }

    /***
     * 获取平台服务费率
     * @author liweifan
     * @param: orderTypeEnum
     * @updateTime 2022/3/31 16:49
     * @return: java.math.BigDecimal
     */
    private BigDecimal getServiceFeeRate(GoodTypeEnum goodTypeEnum) {
        String configValue = "0";
        if (GoodTypeEnum.VIDEO.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.VIDEO_LESSON_SERVICE_FEE);
        } else if (GoodTypeEnum.PRACTICE.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.PRACTICE_SERVICE_FEE);
        } else if (GoodTypeEnum.LIVE.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.LIVE_SERVICE_RATE);
        } else if (GoodTypeEnum.MUSIC.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.MUSIC_SHEET_SERVICE_FEE);
        }
        return new BigDecimal(configValue).divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP);
    }

    /***
     * 获取商品图片
     * @author liweifan
     * @param: orderTypeEnum
     * @updateTime 2022/3/31 16:49
     * @return: java.math.BigDecimal
     */
    private String getGoodUrlByType(GoodTypeEnum goodTypeEnum) {
        String configValue = "";
        if (GoodTypeEnum.VIP.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_VIP);
        } else if (GoodTypeEnum.VIDEO.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_VIDEO);
        } else if (GoodTypeEnum.PRACTICE.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_PRACTICE);
        } else if (GoodTypeEnum.LIVE.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_LIVE);
        } else if (GoodTypeEnum.MUSIC.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_MUSIC);
        } else if (GoodTypeEnum.PINAO_ROOM.equals(goodTypeEnum)) {
            configValue = sysConfigService.findConfigValue(SysConfigConstant.GOOD_LOGO_PIANO_ROOM);
        }
        return configValue;
    }

    private Boolean checkPositiveOrZero(BigDecimal amount) {
        if (null == amount) {
            return false;
        }
        if (amount.doubleValue() < 0) {
            return false;
        }
        return true;
    }

    /***
     * 取消订单
     * @author liweifan
     * @param: userOrder
     * @updateTime 2022/4/13 17:23
     */
    @Transactional(rollbackFor = Exception.class)
    public void doOrderCancel(UserOrderVo userOrder, OrderStatusEnum orderStatus, String reason) {
        //已经完成过的订单，不能再取消
        if (!OrderStatusEnum.WAIT_PAY.equals(userOrder.getStatus())
                && !OrderStatusEnum.PAYING.equals(userOrder.getStatus())) {
            return;
        }
        //状态只能是关闭和失败
        if (!OrderStatusEnum.CLOSE.equals(orderStatus)
                && !OrderStatusEnum.FAIL.equals(orderStatus)) {
            return;
        }

        List<UserOrderDetailVo> orderDetailList = orderDetailService.getOrderDetilListByOrderNo(userOrder.getOrderNo());
        for (UserOrderDetailVo orderDetailVo : orderDetailList) {
            orderDetailVo.setUserId(userOrder.getUserId());
            orderDetailVo.setOrderId(userOrder.getId());
            //调用业务
            Consumer<UserOrderDetailVo> userOrderVoConsumer = orderCancel.get(orderDetailVo.getGoodType());
            if (!Objects.isNull(userOrderVoConsumer)) {
                userOrderVoConsumer.accept(orderDetailVo);
            }
        }
        userOrder.setReason(reason);
        userOrder.setStatus(orderStatus);
        userOrder.setUpdateTime(new Date());
        baseMapper.updateById(userOrder);
    }

    private void orderSuccess(UserOrderVo detail) {
        orderSuccess(detail, null);
    }

    /**
     * 订单完成
     *
     * @author liweifan
     * @param: detail
     * @updateTime 2022/4/13 17:17
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orderSuccess(UserOrderVo detail, PaymentCallBack data) {
        //订单已完成
        if (!OrderStatusEnum.WAIT_PAY.equals(detail.getStatus())
                && !OrderStatusEnum.PAYING.equals(detail.getStatus())) {
            return;
        }
        //修改订单状态
        Date now = new Date();
        detail.setStatus(OrderStatusEnum.PAID);
        detail.setPayTime(now);
        detail.setUpdateTime(now);
        updateById(detail);

        try {
            if (null != data) {
                //更新付款单
                UserOrderPayment orderPayment = orderPaymentService
                        .detailByTransNoOrPaymentNo(data.getId(), data.getPaymentNo());
                if (null != orderPayment) {
                    orderPayment.setTransNo(data.getId());
                    orderPayment.setStatus(TradeStatusEnum.succeeded);
                    orderPayment.setArrivalTime(now);
                    orderPayment.setUpdateTime(now);

                    if (null != data.getExecutePaymentCallBack()) {
                        orderPayment.setBackPayAmt(
                                new BigDecimal(data.getExecutePaymentCallBack().getPayAmt()));

                        BigDecimal feeAmt = new BigDecimal(data.getExecutePaymentCallBack().getFeeAmt());
                        if (BigDecimal.ZERO.compareTo(feeAmt) == 0) {
                            orderPayment.setFeeAmt(orderPayment.getPayAmt().subtract(orderPayment.getBackPayAmt()));
                        } else {
                            orderPayment.setFeeAmt(feeAmt);
                        }
                    }
                    orderPaymentService.updateById(orderPayment);
                }
            }
        } catch (Exception e) {
            log.error("订单完成-更新付款单失败, UserOrderVo is {},PaymentCallBack is {}", JSONObject.toJSONString(detail), JSONObject.toJSONString(data));
            e.printStackTrace();
        }
        //调用业务
        List<UserOrderDetailVo> orderDetailList = orderDetailService.getOrderDetilListByOrderNo(detail.getOrderNo());

        for (UserOrderDetailVo orderDetailVo : orderDetailList) {
            orderDetailVo.setUserId(detail.getUserId());
            orderDetailVo.setOrderId(detail.getId());
            //调用业务
            Consumer<UserOrderDetailVo> userOrderVoConsumer = orderSuccess.get(orderDetailVo.getGoodType());
            if (!Objects.isNull(userOrderVoConsumer)) {
                userOrderVoConsumer.accept(orderDetailVo);
            }
            //清除商家统计缓存
            Long merchId = orderDetailVo.getMerchId();
            if (merchId != null && merchId != 0) {
                redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(merchId)).delete();
            }
        }
        //清除买家统计缓存
        if (ClientEnum.STUDENT.getCode().equals(detail.getPaymentClient())) {
            redissonClient.getBucket(CacheNameEnum.STUDENT_TOTAL.getRedisKey(detail.getUserId())).delete();
        } else if (ClientEnum.TEACHER.getCode().equals(detail.getPaymentClient())) {
            redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(detail.getUserId())).delete();
        }
    }

    private void testOrderSuccess(UserOrderVo detail) {
        if (null == detail || StringUtil.isEmpty(detail.getOrderNo())) {
            return;
        }
        //关闭当前正在支付的订单
        orderPaymentService.closePaymentAndReqOpen(detail.getOrderNo(), "交易取消");

        UserOrderPayment orderPayment = new UserOrderPayment();
        OpenEnum openType = OpenEnum.valueOf(paymentProperties.getOpenType());
        orderPayment.setOpenType(openType);

        orderPayment.setOrderNo(detail.getOrderNo());

        //支付订单号生成
        Long paymentNo = idGeneratorService.generatorId("paymentNo");
        orderPayment.setPaymentNo(paymentNo.toString());

        orderPayment.setPayAmt(detail.getActualPrice());
        orderPayment.setStatus(TradeStatusEnum.succeeded);

        //保存新的付款单
        orderPaymentService.save(orderPayment);
        orderSuccess(detail, null);
    }

}